@@ -287,7 +287,18 @@ fr_command_cfile_add (FrCommand  *comm,
 		fr_process_add_arg (comm->process, filename);
 		fr_process_end_command (comm->process);
 		compressed_filename = g_strconcat (filename, ".rz", NULL);
-	} else {
+	}
+	else if (_g_mime_type_matches (archive->mime_type, "application/x-lz4")) {
+		compressed_filename = g_strconcat (filename, ".lz4", NULL);
+		fr_process_begin_command (comm->process, "lz4");
+		fr_process_set_working_dir (comm->process, temp_dir);
+		fr_process_add_arg (comm->process, "-f");
+		fr_process_add_arg (comm->process, "-z");
+		fr_process_add_arg (comm->process, filename);
+		fr_process_add_arg (comm->process, compressed_filename);
+		fr_process_end_command (comm->process);
+	}
+	else {
 		g_warning ("Unhandled mime type: '%s'", archive->mime_type);
 		g_warn_if_reached ();
 		g_free (temp_file);
@@ -361,8 +372,11 @@ fr_command_cfile_extract (FrCommand  *comm,
 
 	/* uncompress the file */
 
+	uncompr_file = _g_path_remove_first_extension (temp_file);
+
 	if (_g_mime_type_matches (archive->mime_type, "application/x-gzip")) {
 		fr_process_begin_command (comm->process, "gzip");
+		fr_process_set_working_dir (comm->process, temp_dir);
 		fr_process_add_arg (comm->process, "-f");
 		fr_process_add_arg (comm->process, "-d");
 		fr_process_add_arg (comm->process, "-n");
@@ -371,6 +385,7 @@ fr_command_cfile_extract (FrCommand  *comm,
 	}
 	else if (_g_mime_type_matches (archive->mime_type, "application/x-bzip")) {
 		fr_process_begin_command (comm->process, "bzip2");
+		fr_process_set_working_dir (comm->process, temp_dir);
 		fr_process_add_arg (comm->process, "-f");
 		fr_process_add_arg (comm->process, "-d");
 		fr_process_add_arg (comm->process, temp_file);
@@ -384,12 +399,14 @@ fr_command_cfile_extract (FrCommand  *comm,
 		}
 		else
 			fr_process_begin_command (comm->process, "uncompress");
+		fr_process_set_working_dir (comm->process, temp_dir);
 		fr_process_add_arg (comm->process, "-f");
 		fr_process_add_arg (comm->process, temp_file);
 		fr_process_end_command (comm->process);
 	}
 	else if (_g_mime_type_matches (archive->mime_type, "application/x-lzip")) {
 		fr_process_begin_command (comm->process, "lzip");
+		fr_process_set_working_dir (comm->process, temp_dir);
 		fr_process_add_arg (comm->process, "-f");
 		fr_process_add_arg (comm->process, "-d");
 		fr_process_add_arg (comm->process, temp_file);
@@ -397,6 +414,7 @@ fr_command_cfile_extract (FrCommand  *comm,
 	}
 	else if (_g_mime_type_matches (archive->mime_type, "application/x-lzma")) {
 		fr_process_begin_command (comm->process, "lzma");
+		fr_process_set_working_dir (comm->process, temp_dir);
 		fr_process_add_arg (comm->process, "-f");
 		fr_process_add_arg (comm->process, "-d");
 		fr_process_add_arg (comm->process, temp_file);
@@ -404,6 +422,7 @@ fr_command_cfile_extract (FrCommand  *comm,
 	}
 	else if (_g_mime_type_matches (archive->mime_type, "application/x-xz")) {
 		fr_process_begin_command (comm->process, "xz");
+		fr_process_set_working_dir (comm->process, temp_dir);
 		fr_process_add_arg (comm->process, "-f");
 		fr_process_add_arg (comm->process, "-d");
 		fr_process_add_arg (comm->process, temp_file);
@@ -420,16 +439,24 @@ fr_command_cfile_extract (FrCommand  *comm,
 	}
 	else if (_g_mime_type_matches (archive->mime_type, "application/x-rzip")) {
 		fr_process_begin_command (comm->process, "rzip");
+		fr_process_set_working_dir (comm->process, temp_dir);
 		fr_process_add_arg (comm->process, "-f");
 		fr_process_add_arg (comm->process, "-d");
 		fr_process_add_arg (comm->process, temp_file);
 		fr_process_end_command (comm->process);
 	}
+	else if (_g_mime_type_matches (archive->mime_type, "application/x-lz4")) {
+		fr_process_begin_command (comm->process, "lz4");
+		fr_process_set_working_dir (comm->process, temp_dir);
+		fr_process_add_arg (comm->process, "-f");
+		fr_process_add_arg (comm->process, "-d");
+		fr_process_add_arg (comm->process, temp_file);
+		fr_process_add_arg (comm->process, uncompr_file);
+		fr_process_end_command (comm->process);
+	}
 
 	/* copy uncompress file to the dest dir */
 
-	uncompr_file = _g_path_remove_first_extension (temp_file);
-
 	compr_file = get_uncompressed_name_from_archive (comm, comm->filename);
 	if (compr_file == NULL)
 		compr_file = _g_path_remove_first_extension (_g_path_get_basename (comm->filename));
@@ -463,6 +490,7 @@ fr_command_cfile_extract (FrCommand  *comm,
 const char *cfile_mime_type[] = { "application/x-gzip",
 				  "application/x-bzip",
 				  "application/x-compress",
+				  "application/x-lz4",
 				  "application/x-lzip",
 				  "application/x-lzma",
 				  "application/x-lzop",
@@ -521,7 +549,10 @@ fr_command_cfile_get_capabilities (FrArchive  *archive,
 		if (_g_program_is_available ("rzip", check_command))
 			capabilities |= FR_ARCHIVE_CAN_READ_WRITE;
 	}
-
+	else if (_g_mime_type_matches (mime_type, "application/x-lz4")) {
+		if (_g_program_is_available ("lz4", check_command))
+			capabilities |= FR_ARCHIVE_CAN_READ_WRITE;
+	}
 	return capabilities;
 }
 
@@ -546,6 +577,8 @@ fr_command_cfile_get_packages (FrArchive  *archive,
 		return PACKAGES ("lzop");
 	else if (_g_mime_type_matches (mime_type, "application/x-rzip"))
 		return PACKAGES ("rzip");
+	else if (_g_mime_type_matches (mime_type, "application/x-lz4"))
+		return PACKAGES ("lz4");
 
 	return NULL;
 }
